@@ -16,13 +16,14 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include	"sfconfig.h"
+
 #include	<stdio.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
 
 #include	"sndfile.h"
-#include	"sfconfig.h"
 #include	"sfendian.h"
 #include	"common.h"
 #include	"float_cast.h"
@@ -101,7 +102,7 @@ mat4_open	(SF_PRIVATE *psf)
 		psf->write_header = mat4_write_header ;
 		} ;
 
-	psf->close = mat4_close ;
+	psf->container_close = mat4_close ;
 
 	psf->blockwidth = psf->bytewidth * psf->sf.channels ;
 
