@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2003,2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2003-2006 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,7 +16,7 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include "config.h"
+#include "sfconfig.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -105,7 +105,7 @@ xi_open	(SF_PRIVATE *psf)
 		psf->write_header = xi_write_header ;
 		} ;
 
-	psf->close = xi_close ;
+	psf->container_close = xi_close ;
 	psf->seek = dpcm_seek ;
 
 	psf->sf.seekable = SF_FALSE ;
@@ -143,20 +143,20 @@ static sf_count_t dpcm_read_dsc2i (SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
 static sf_count_t dpcm_read_dsc2f (SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
 static sf_count_t dpcm_read_dsc2d (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
 
-static sf_count_t dpcm_write_s2dsc (SF_PRIVATE *psf, short *ptr, sf_count_t len) ;
-static sf_count_t dpcm_write_i2dsc (SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
-static sf_count_t dpcm_write_f2dsc (SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
-static sf_count_t dpcm_write_d2dsc (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
+static sf_count_t dpcm_write_s2dsc (SF_PRIVATE *psf, const short *ptr, sf_count_t len) ;
+static sf_count_t dpcm_write_i2dsc (SF_PRIVATE *psf, const int *ptr, sf_count_t len) ;
+static sf_count_t dpcm_write_f2dsc (SF_PRIVATE *psf, const float *ptr, sf_count_t len) ;
+static sf_count_t dpcm_write_d2dsc (SF_PRIVATE *psf, const double *ptr, sf_count_t len) ;
 
 static sf_count_t dpcm_read_dles2s (SF_PRIVATE *psf, short *ptr, sf_count_t len) ;
 static sf_count_t dpcm_read_dles2i (SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
 static sf_count_t dpcm_read_dles2f (SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
 static sf_count_t dpcm_read_dles2d (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
 
-static sf_count_t dpcm_write_s2dles (SF_PRIVATE *psf, short *ptr, sf_count_t len) ;
-static sf_count_t dpcm_write_i2dles (SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
-static sf_count_t dpcm_write_f2dles (SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
-static sf_count_t dpcm_write_d2dles (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
+static sf_count_t dpcm_write_s2dles (SF_PRIVATE *psf, const short *ptr, sf_count_t len) ;
+static sf_count_t dpcm_write_i2dles (SF_PRIVATE *psf, const int *ptr, sf_count_t len) ;
+static sf_count_t dpcm_write_f2dles (SF_PRIVATE *psf, const float *ptr, sf_count_t len) ;
+static sf_count_t dpcm_write_d2dles (SF_PRIVATE *psf, const double *ptr, sf_count_t len) ;
 
 static int
 dpcm_init (SF_PRIVATE *psf)
@@ -226,7 +226,7 @@ dpcm_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 
 	if (psf->datalength < 0 || psf->dataoffset < 0)
 	{	psf->error = SFE_BAD_SEEK ;
-		return	((sf_count_t) -1) ;
+		return	PSF_SEEK_ERROR ;
 		} ;
 
 	if (offset == 0)
@@ -237,31 +237,31 @@ dpcm_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 
 	if (offset < 0 || offset > psf->sf.frames)
 	{	psf->error = SFE_BAD_SEEK ;
-		return	((sf_count_t) -1) ;
+		return	PSF_SEEK_ERROR ;
 		} ;
 
 	if (mode != SFM_READ)
 	{	/* What to do about write??? */
 		psf->error = SFE_BAD_SEEK ;
-		return	((sf_count_t) -1) ;
+		return	PSF_SEEK_ERROR ;
 		} ;
 
 	psf_fseek (psf, psf->dataoffset, SEEK_SET) ;
 
 	if ((psf->sf.format & SF_FORMAT_SUBMASK) == SF_FORMAT_DPCM_16)
 	{	total = offset ;
-		bufferlen = sizeof (psf->buffer) / sizeof (short) ;
+		bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 		while (total > 0)
 		{	len = (total > bufferlen) ? bufferlen : total ;
-			total -= dpcm_read_dles2s (psf, (short *) psf->buffer, len) ;
+			total -= dpcm_read_dles2s (psf, psf->u.sbuf, len) ;
 			} ;
 		}
 	else
 	{	total = offset ;
-		bufferlen = sizeof (psf->buffer) / sizeof (short) ;
+		bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 		while (total > 0)
 		{	len = (total > bufferlen) ? bufferlen : total ;
-			total -= dpcm_read_dsc2s (psf, (short *) psf->buffer, len) ;
+			total -= dpcm_read_dsc2s (psf, psf->u.sbuf, len) ;
 			} ;
 		} ;
 
@@ -303,11 +303,10 @@ xi_write_header (SF_PRIVATE *psf, int calc_length)
 	/* Jump volume loop (3 bytes), pan loop (3), envelope flags (3), vibrato (3)
 	** fade out (2), 22 unknown bytes, and then write sample_count (2 bytes).
 	*/
-	psf_binheader_writef (psf, "ez2z2", (size_t) (4 * 3), 0x1234, (size_t) 22, 1) ;
+	psf_binheader_writef (psf, "ez2z2", (size_t) (4 * 3), 0x1234, make_size_t (22), 1) ;
 
-psf->sf.frames = 12 ;
-pxi->loop_begin = 0 ;
-pxi->loop_end = 0 ;
+	pxi->loop_begin = 0 ;
+	pxi->loop_end = 0 ;
 
 	psf_binheader_writef (psf, "et844", psf->sf.frames, pxi->loop_begin, pxi->loop_end) ;
 
@@ -389,6 +388,9 @@ xi_read_header (SF_PRIVATE *psf)
 	if (sample_count > MAX_XI_SAMPLES)
 		return SFE_XI_EXCESS_SAMPLES ;
 
+	if (psf->instrument == NULL && (psf->instrument = psf_instrument_alloc ()) == NULL)
+		return SFE_MALLOC_FAILED ;
+
 	/* Log all data for each sample. */
 	for (k = 0 ; k < sample_count ; k++)
 	{	psf_binheader_readf (psf, "e444", &(sample_sizes [k]), &loop_begin, &loop_end) ;
@@ -397,7 +399,12 @@ xi_read_header (SF_PRIVATE *psf)
 		psf_binheader_readf (psf, "bb", buffer, 6, name, 22) ;
 		name [21] = 0 ;
 
-		psf_log_printf (psf, "Sample #%d\n  name    : %s\n  size    : %d\n", k + 1, name, sample_sizes [k]) ;
+		psf_log_printf (psf, "Sample #%d\n  name    : %s\n", k + 1, name) ;
+
+		psf_log_printf (psf, "  size    : %d\n", sample_sizes [k]) ;
+
+
+
 		psf_log_printf (psf, "  loop\n    begin : %d\n    end   : %d\n", loop_begin, loop_end) ;
 
 		psf_log_printf (psf, "  volume  : %u\n  f. tune : %d\n  flags   : 0x%02X ",
@@ -453,8 +460,6 @@ xi_read_header (SF_PRIVATE *psf)
  	if (psf_fseek (psf, psf->dataoffset, SEEK_SET) != psf->dataoffset)
 		return SFE_BAD_SEEK ;
 
-	psf->close = xi_close ;
-
 	psf->endian = SF_ENDIAN_LITTLE ;
 	psf->sf.channels = 1 ; /* Always mono */
 	psf->sf.samplerate = 44100 ; /* Always */
@@ -464,6 +469,11 @@ xi_read_header (SF_PRIVATE *psf)
 	if (! psf->sf.frames && psf->blockwidth)
 		psf->sf.frames = (psf->filelength - psf->dataoffset) / psf->blockwidth ;
 
+	psf->instrument->basenote = 0 ;
+	psf->instrument->gain = 1 ;
+	psf->instrument->velocity_lo = psf->instrument->key_lo = 0 ;
+	psf->instrument->velocity_hi = psf->instrument->key_hi = 127 ;
+
 	return 0 ;
 } /* xi_read_header */
 
@@ -483,22 +493,23 @@ static void dles2d_array (XI_PRIVATE *pxi, short *src, int count, double *dest,
 static sf_count_t
 dpcm_read_dsc2s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, readcount, thisread ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
 	if ((pxi = psf->fdata) == NULL)
 		return 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (signed char) ;
+	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (signed char), readcount, psf) ;
-		dsc2s_array (pxi, (signed char*) (psf->buffer), thisread, ptr + total) ;
-		total += thisread ;
-		len -= thisread ;
-		if (thisread < readcount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.scbuf, sizeof (signed char), bufferlen, psf) ;
+		dsc2s_array (pxi, psf->u.scbuf, readcount, ptr + total) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -507,22 +518,23 @@ dpcm_read_dsc2s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 static sf_count_t
 dpcm_read_dsc2i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, readcount, thisread ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
 	if ((pxi = psf->fdata) == NULL)
 		return 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (signed char) ;
+	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (signed char), readcount, psf) ;
-		dsc2i_array (pxi, (signed char*) (psf->buffer), thisread, ptr + total) ;
-		total += thisread ;
-		len -= thisread ;
-		if (thisread < readcount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.scbuf, sizeof (signed char), bufferlen, psf) ;
+		dsc2i_array (pxi, psf->u.scbuf, readcount, ptr + total) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -531,7 +543,7 @@ dpcm_read_dsc2i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 static sf_count_t
 dpcm_read_dsc2f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, readcount, thisread ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
@@ -540,16 +552,17 @@ dpcm_read_dsc2f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 
 	normfact = (psf->norm_float == SF_TRUE) ? 1.0 / ((float) 0x80) : 1.0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (signed char) ;
+	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (signed char), readcount, psf) ;
-		dsc2f_array (pxi, (signed char*) (psf->buffer), thisread, ptr + total, normfact) ;
-		total += thisread ;
-		len -= thisread ;
-		if (thisread < readcount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.scbuf, sizeof (signed char), bufferlen, psf) ;
+		dsc2f_array (pxi, psf->u.scbuf, readcount, ptr + total, normfact) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -558,7 +571,7 @@ dpcm_read_dsc2f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 static sf_count_t
 dpcm_read_dsc2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, readcount, thisread ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 	double		normfact ;
 
@@ -567,16 +580,17 @@ dpcm_read_dsc2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 	normfact = (psf->norm_double == SF_TRUE) ? 1.0 / ((double) 0x80) : 1.0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (signed char) ;
+	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (signed char), readcount, psf) ;
-		dsc2d_array (pxi, (signed char*) (psf->buffer), thisread, ptr + total, normfact) ;
-		total += thisread ;
-		len -= thisread ;
-		if (thisread < readcount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.scbuf, sizeof (signed char), bufferlen, psf) ;
+		dsc2d_array (pxi, psf->u.scbuf, readcount, ptr + total, normfact) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -588,22 +602,23 @@ dpcm_read_dsc2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 static sf_count_t
 dpcm_read_dles2s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, readcount, thisread ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
 	if ((pxi = psf->fdata) == NULL)
 		return 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (short) ;
+	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (short), readcount, psf) ;
-		dles2s_array (pxi, (short*) (psf->buffer), thisread, ptr + total) ;
-		total += thisread ;
-		len -= thisread ;
-		if (thisread < readcount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.sbuf, sizeof (short), bufferlen, psf) ;
+		dles2s_array (pxi, psf->u.sbuf, readcount, ptr + total) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -612,22 +627,23 @@ dpcm_read_dles2s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 static sf_count_t
 dpcm_read_dles2i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, readcount, thisread ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
 	if ((pxi = psf->fdata) == NULL)
 		return 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (short) ;
+	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (short), readcount, psf) ;
-		dles2i_array (pxi, (short*) (psf->buffer), thisread, ptr + total) ;
-		total += thisread ;
-		len -= thisread ;
-		if (thisread < readcount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.sbuf, sizeof (short), bufferlen, psf) ;
+		dles2i_array (pxi, psf->u.sbuf, readcount, ptr + total) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -636,7 +652,7 @@ dpcm_read_dles2i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 static sf_count_t
 dpcm_read_dles2f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, readcount, thisread ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
@@ -645,16 +661,17 @@ dpcm_read_dles2f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 
 	normfact = (psf->norm_float == SF_TRUE) ? 1.0 / ((float) 0x8000) : 1.0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (short) ;
+	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (short), readcount, psf) ;
-		dles2f_array (pxi, (short*) (psf->buffer), thisread, ptr + total, normfact) ;
-		total += thisread ;
-		len -= thisread ;
-		if (thisread < readcount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.sbuf, sizeof (short), bufferlen, psf) ;
+		dles2f_array (pxi, psf->u.sbuf, readcount, ptr + total, normfact) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -663,7 +680,7 @@ dpcm_read_dles2f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 static sf_count_t
 dpcm_read_dles2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, readcount, thisread ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 	double		normfact ;
 
@@ -672,16 +689,17 @@ dpcm_read_dles2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 	normfact = (psf->norm_double == SF_TRUE) ? 1.0 / ((double) 0x8000) : 1.0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (short) ;
+	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (short), readcount, psf) ;
-		dles2d_array (pxi, (short*) (psf->buffer), thisread, ptr + total, normfact) ;
-		total += thisread ;
-		len -= thisread ;
-		if (thisread < readcount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.sbuf, sizeof (short), bufferlen, psf) ;
+		dles2d_array (pxi, psf->u.sbuf, readcount, ptr + total, normfact) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -690,69 +708,71 @@ dpcm_read_dles2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 /*==============================================================================
 */
 
-static void s2dsc_array (XI_PRIVATE *pxi, short *src, signed char *dest, int count) ;
-static void i2dsc_array (XI_PRIVATE *pxi, int *src, signed char *dest, int count) ;
-static void f2dsc_array (XI_PRIVATE *pxi, float *src, signed char *dest, int count, float normfact) ;
-static void d2dsc_array (XI_PRIVATE *pxi, double *src, signed char *dest, int count, double normfact) ;
+static void s2dsc_array (XI_PRIVATE *pxi, const short *src, signed char *dest, int count) ;
+static void i2dsc_array (XI_PRIVATE *pxi, const int *src, signed char *dest, int count) ;
+static void f2dsc_array (XI_PRIVATE *pxi, const float *src, signed char *dest, int count, float normfact) ;
+static void d2dsc_array (XI_PRIVATE *pxi, const double *src, signed char *dest, int count, double normfact) ;
 
-static void	s2dles_array (XI_PRIVATE *pxi, short *src, short *dest, int count) ;
-static void i2dles_array (XI_PRIVATE *pxi, int *src, short *dest, int count) ;
-static void f2dles_array (XI_PRIVATE *pxi, float *src, short *dest, int count, float normfact) ;
-static void d2dles_array (XI_PRIVATE *pxi, double *src, short *dest, int count, double normfact) ;
+static void	s2dles_array (XI_PRIVATE *pxi, const short *src, short *dest, int count) ;
+static void i2dles_array (XI_PRIVATE *pxi, const int *src, short *dest, int count) ;
+static void f2dles_array (XI_PRIVATE *pxi, const float *src, short *dest, int count, float normfact) ;
+static void d2dles_array (XI_PRIVATE *pxi, const double *src, short *dest, int count, double normfact) ;
 
 
 static sf_count_t
-dpcm_write_s2dsc (SF_PRIVATE *psf, short *ptr, sf_count_t len)
+dpcm_write_s2dsc (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, writecount, thiswrite ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
 	if ((pxi = psf->fdata) == NULL)
 		return 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (signed char) ;
+	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		s2dsc_array (pxi, ptr + total, (signed char*) (psf->buffer), writecount) ;
-		thiswrite = psf_fwrite (psf->buffer, sizeof (signed char), writecount, psf) ;
-		total += thiswrite ;
-		len -= thiswrite ;
-		if (thiswrite < writecount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		s2dsc_array (pxi, ptr + total, psf->u.scbuf, bufferlen) ;
+		writecount = psf_fwrite (psf->u.scbuf, sizeof (signed char), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* dpcm_write_s2dsc */
 
 static sf_count_t
-dpcm_write_i2dsc (SF_PRIVATE *psf, int *ptr, sf_count_t len)
+dpcm_write_i2dsc (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, writecount, thiswrite ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
 	if ((pxi = psf->fdata) == NULL)
 		return 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (signed char) ;
+	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		i2dsc_array (pxi, ptr + total, (signed char*) (psf->buffer), writecount) ;
-		thiswrite = psf_fwrite (psf->buffer, sizeof (signed char), writecount, psf) ;
-		total += thiswrite ;
-		len -= thiswrite ;
-		if (thiswrite < writecount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		i2dsc_array (pxi, ptr + total, psf->u.scbuf, bufferlen) ;
+		writecount = psf_fwrite (psf->u.scbuf, sizeof (signed char), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* dpcm_write_i2dsc */
 
 static sf_count_t
-dpcm_write_f2dsc (SF_PRIVATE *psf, float *ptr, sf_count_t len)
+dpcm_write_f2dsc (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, writecount, thiswrite ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
@@ -761,25 +781,26 @@ dpcm_write_f2dsc (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 * 0x7F) : 1.0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (signed char) ;
+	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		f2dsc_array (pxi, ptr + total, (signed char*) (psf->buffer), writecount, normfact) ;
-		thiswrite = psf_fwrite (psf->buffer, sizeof (signed char), writecount, psf) ;
-		total += thiswrite ;
-		len -= thiswrite ;
-		if (thiswrite < writecount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		f2dsc_array (pxi, ptr + total, psf->u.scbuf, bufferlen, normfact) ;
+		writecount = psf_fwrite (psf->u.scbuf, sizeof (signed char), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* dpcm_write_f2dsc */
 
 static sf_count_t
-dpcm_write_d2dsc (SF_PRIVATE *psf, double *ptr, sf_count_t len)
+dpcm_write_d2dsc (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, writecount, thiswrite ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 	double		normfact ;
 
@@ -788,16 +809,17 @@ dpcm_write_d2dsc (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 * 0x7F) : 1.0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (signed char) ;
+	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		d2dsc_array (pxi, ptr + total, (signed char*) (psf->buffer), writecount, normfact) ;
-		thiswrite = psf_fwrite (psf->buffer, sizeof (signed char), writecount, psf) ;
-		total += thiswrite ;
-		len -= thiswrite ;
-		if (thiswrite < writecount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		d2dsc_array (pxi, ptr + total, psf->u.scbuf, bufferlen, normfact) ;
+		writecount = psf_fwrite (psf->u.scbuf, sizeof (signed char), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
+		len -= writecount ;
 		} ;
 
 	return total ;
@@ -805,57 +827,59 @@ dpcm_write_d2dsc (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 
 static sf_count_t
-dpcm_write_s2dles (SF_PRIVATE *psf, short *ptr, sf_count_t len)
+dpcm_write_s2dles (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, writecount, thiswrite ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
 	if ((pxi = psf->fdata) == NULL)
 		return 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (short) ;
+	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		s2dles_array (pxi, ptr + total, (short*) (psf->buffer), writecount) ;
-		thiswrite = psf_fwrite (psf->buffer, sizeof (short), writecount, psf) ;
-		total += thiswrite ;
-		len -= thiswrite ;
-		if (thiswrite < writecount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		s2dles_array (pxi, ptr + total, psf->u.sbuf, bufferlen) ;
+		writecount = psf_fwrite (psf->u.sbuf, sizeof (short), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* dpcm_write_s2dles */
 
 static sf_count_t
-dpcm_write_i2dles (SF_PRIVATE *psf, int *ptr, sf_count_t len)
+dpcm_write_i2dles (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, writecount, thiswrite ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
 	if ((pxi = psf->fdata) == NULL)
 		return 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (short) ;
+	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		i2dles_array (pxi, ptr + total, (short*) (psf->buffer), writecount) ;
-		thiswrite = psf_fwrite (psf->buffer, sizeof (short), writecount, psf) ;
-		total += thiswrite ;
-		len -= thiswrite ;
-		if (thiswrite < writecount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		i2dles_array (pxi, ptr + total, psf->u.sbuf, bufferlen) ;
+		writecount = psf_fwrite (psf->u.sbuf, sizeof (short), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* dpcm_write_i2dles */
 
 static sf_count_t
-dpcm_write_f2dles (SF_PRIVATE *psf, float *ptr, sf_count_t len)
+dpcm_write_f2dles (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, writecount, thiswrite ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
@@ -864,25 +888,26 @@ dpcm_write_f2dles (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 * 0x7FFF) : 1.0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (short) ;
+	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		f2dles_array (pxi, ptr + total, (short*) (psf->buffer), writecount, normfact) ;
-		thiswrite = psf_fwrite (psf->buffer, sizeof (short), writecount, psf) ;
-		total += thiswrite ;
-		len -= thiswrite ;
-		if (thiswrite < writecount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		f2dles_array (pxi, ptr + total, psf->u.sbuf, bufferlen, normfact) ;
+		writecount = psf_fwrite (psf->u.sbuf, sizeof (short), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* dpcm_write_f2dles */
 
 static sf_count_t
-dpcm_write_d2dles (SF_PRIVATE *psf, double *ptr, sf_count_t len)
+dpcm_write_d2dles (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 {	XI_PRIVATE	*pxi ;
-	int			bufferlen, writecount, thiswrite ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 	double		normfact ;
 
@@ -891,16 +916,17 @@ dpcm_write_d2dles (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 * 0x7FFF) : 1.0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (short) ;
+	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		d2dles_array (pxi, ptr + total, (short*) (psf->buffer), writecount, normfact) ;
-		thiswrite = psf_fwrite (psf->buffer, sizeof (short), writecount, psf) ;
-		total += thiswrite ;
-		len -= thiswrite ;
-		if (thiswrite < writecount)
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		d2dles_array (pxi, ptr + total, psf->u.sbuf, bufferlen, normfact) ;
+		writecount = psf_fwrite (psf->u.sbuf, sizeof (short), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
+		len -= writecount ;
 		} ;
 
 	return total ;
@@ -974,7 +1000,7 @@ dsc2d_array (XI_PRIVATE *pxi, signed char *src, int count, double *dest, double
 */
 
 static void
-s2dsc_array (XI_PRIVATE *pxi, short *src, signed char *dest, int count)
+s2dsc_array (XI_PRIVATE *pxi, const short *src, signed char *dest, int count)
 {	signed char	last_val, current ;
 	int			k ;
 
@@ -990,7 +1016,7 @@ s2dsc_array (XI_PRIVATE *pxi, short *src, signed char *dest, int count)
 } /* s2dsc_array */
 
 static void
-i2dsc_array (XI_PRIVATE *pxi, int *src, signed char *dest, int count)
+i2dsc_array (XI_PRIVATE *pxi, const int *src, signed char *dest, int count)
 {	signed char	last_val, current ;
 	int			k ;
 
@@ -1006,7 +1032,7 @@ i2dsc_array (XI_PRIVATE *pxi, int *src, signed char *dest, int count)
 } /* i2dsc_array */
 
 static void
-f2dsc_array (XI_PRIVATE *pxi, float *src, signed char *dest, int count, float normfact)
+f2dsc_array (XI_PRIVATE *pxi, const float *src, signed char *dest, int count, float normfact)
 {	signed char	last_val, current ;
 	int			k ;
 
@@ -1022,7 +1048,7 @@ f2dsc_array (XI_PRIVATE *pxi, float *src, signed char *dest, int count, float no
 } /* f2dsc_array */
 
 static void
-d2dsc_array (XI_PRIVATE *pxi, double *src, signed char *dest, int count, double normfact)
+d2dsc_array (XI_PRIVATE *pxi, const double *src, signed char *dest, int count, double normfact)
 {	signed char	last_val, current ;
 	int			k ;
 
@@ -1104,7 +1130,7 @@ dles2d_array (XI_PRIVATE *pxi, short *src, int count, double *dest, double normf
 */
 
 static void
-s2dles_array (XI_PRIVATE *pxi, short *src, short *dest, int count)
+s2dles_array (XI_PRIVATE *pxi, const short *src, short *dest, int count)
 {	short	diff, last_val ;
 	int		k ;
 
@@ -1120,7 +1146,7 @@ s2dles_array (XI_PRIVATE *pxi, short *src, short *dest, int count)
 } /* s2dles_array */
 
 static void
-i2dles_array (XI_PRIVATE *pxi, int *src, short *dest, int count)
+i2dles_array (XI_PRIVATE *pxi, const int *src, short *dest, int count)
 {	short	diff, last_val ;
 	int		k ;
 
@@ -1136,7 +1162,7 @@ i2dles_array (XI_PRIVATE *pxi, int *src, short *dest, int count)
 } /* i2dles_array */
 
 static void
-f2dles_array (XI_PRIVATE *pxi, float *src, short *dest, int count, float normfact)
+f2dles_array (XI_PRIVATE *pxi, const float *src, short *dest, int count, float normfact)
 {	short	diff, last_val, current ;
 	int		k ;
 
@@ -1153,7 +1179,7 @@ f2dles_array (XI_PRIVATE *pxi, float *src, short *dest, int count, float normfac
 } /* f2dles_array */
 
 static void
-d2dles_array (XI_PRIVATE *pxi, double *src, short *dest, int count, double normfact)
+d2dles_array (XI_PRIVATE *pxi, const double *src, short *dest, int count, double normfact)
 {	short	diff, last_val, current ;
 	int		k ;
 
